@@ -20,5 +20,7 @@
 """This module provides access to all submodules."""
 from pymovements import datasets  # noqa: F401
 from pymovements import events  # noqa: F401
+from pymovements import gaze  # noqa: F401
+from pymovements import plotting  # noqa: F401
 from pymovements import synthetic  # noqa: F401
 from pymovements import utils  # noqa: F401
