@@ -27,9 +27,11 @@ from typing import Any
 import polars as pl
 from tqdm.auto import tqdm
 
-from pymovements.base import Experiment
-from pymovements.events.events import Event
+from pymovements.events.event_processing import EventGazeProcessor
+from pymovements.events.events import EventDataFrame
 from pymovements.events.events import EventDetectionCallable
+from pymovements.gaze import GazeDataFrame
+from pymovements.gaze.experiment import Experiment
 from pymovements.utils.paths import match_filepaths
 
 
@@ -88,8 +90,8 @@ class Dataset:
             the user to keep the event data separate from the original raw data. Default: `events`
         """
         self.fileinfo: pl.DataFrame = pl.DataFrame()
-        self.gaze: list[pl.DataFrame] = []
-        self.events: list[pl.DataFrame] = []
+        self.gaze: list[GazeDataFrame] = []
+        self.events: list[EventDataFrame] = []
 
         self._root = Path(root)
         self.dataset_dirname = dataset_dirname
@@ -120,6 +122,7 @@ class Dataset:
             subset: None | dict[str, float | int | str | list[float | int | str]] = None,
             events_dirname: str | None = None,
             preprocessed_dirname: str | None = None,
+            extension: str = 'feather',
     ):
         """Parse file information and load all gaze files.
 
@@ -143,6 +146,9 @@ class Dataset:
             :py:meth:`pymovements.Dataset.path`.
             This argument is used only for this single call and does not alter
             :py:meth:`pymovements.Dataset.preprocessed_rootpath`.
+        extension:
+            Specifies the file format for loading data. Valid options are: `csv`, `feather`.
+            :Default: `feather`.
 
         The parsed file information is assigned to the `fileinfo` attribute.
         All gaze files will be loaded as dataframes and assigned to the `gaze` attribute.
@@ -151,10 +157,14 @@ class Dataset:
         self.fileinfo = self.take_subset(fileinfo=fileinfo, subset=subset)
         self.gaze = self.load_gaze_files(
             preprocessed=preprocessed, preprocessed_dirname=preprocessed_dirname,
+            extension=extension,
         )
 
         if events:
-            self.events = self.load_event_files(events_dirname=events_dirname)
+            self.events = self.load_event_files(
+                events_dirname=events_dirname,
+                extension=extension,
+            )
 
     def infer_fileinfo(self) -> pl.DataFrame:
         """Infer information from filepaths and filenames.
@@ -182,7 +192,7 @@ class Dataset:
             raise RuntimeError(f'no matching files found in {self.raw_rootpath}')
 
         # Create dataframe from all fileinfo records.
-        fileinfo_df = pl.from_dicts(dicts=fileinfo_dicts, infer_schema_length=1)
+        fileinfo_df = pl.from_dicts(data=fileinfo_dicts, infer_schema_length=1)
         fileinfo_df = fileinfo_df.sort(by='filepath')
 
         fileinfo_df = fileinfo_df.with_columns([
@@ -261,7 +271,8 @@ class Dataset:
             self,
             preprocessed: bool = False,
             preprocessed_dirname: str | None = None,
-    ) -> list[pl.DataFrame]:
+            extension: str = 'feather',
+    ) -> list[GazeDataFrame]:
         """Load all available gaze data files.
 
         Parameters
@@ -273,10 +284,13 @@ class Dataset:
             :py:meth:`pymovements.Dataset.path`.
             This argument is used only for this single call and does not alter
             :py:meth:`pymovements.Dataset.preprocessed_rootpath`.
+        extension:
+            Specifies the file format for loading data. Valid options are: `csv`, `feather`.
+            :Default: `feather`.
 
         Returns
         -------
-        list[pl.DataFrame]
+        list[GazeDataFrame]
             List of gaze dataframes.
 
         Raises
@@ -288,7 +302,7 @@ class Dataset:
         """
         self._check_fileinfo()
 
-        gaze_dfs: list[pl.DataFrame] = []
+        gaze_dfs: list[GazeDataFrame] = []
 
         # Read gaze files from fileinfo attribute.
         for fileinfo in tqdm(self.fileinfo.to_dicts()):
@@ -298,10 +312,14 @@ class Dataset:
             if preprocessed:
                 filepath = self._raw_to_preprocessed_filepath(
                     filepath, preprocessed_dirname=preprocessed_dirname,
+                    extension=extension,
                 )
 
             if filepath.suffix == '.csv':
-                gaze_df = pl.read_csv(filepath, **self._custom_read_kwargs)
+                if preprocessed:
+                    gaze_df = pl.read_csv(filepath)
+                else:
+                    gaze_df = pl.read_csv(filepath, **self._custom_read_kwargs)
             elif filepath.suffix == '.feather':
                 gaze_df = pl.read_ipc(filepath)
             else:
@@ -310,11 +328,15 @@ class Dataset:
             # Add fileinfo columns to dataframe.
             gaze_df = self._add_fileinfo(gaze_df, fileinfo)
 
-            gaze_dfs.append(gaze_df)
+            gaze_dfs.append(GazeDataFrame(gaze_df, experiment=self.experiment))
 
         return gaze_dfs
 
-    def load_event_files(self, events_dirname: str | None = None) -> list[pl.DataFrame]:
+    def load_event_files(
+        self,
+        events_dirname: str | None = None,
+        extension: str = 'feather',
+    ) -> list[EventDataFrame]:
         """Load all available event files.
 
         Parameters
@@ -324,33 +346,51 @@ class Dataset:
             :py:meth:`pymovements.Dataset.path`.
             This argument is used only for this single call and does not alter
             :py:meth:`pymovements.Dataset.events_rootpath`.
+        extension:
+            Specifies the file format for loading data. Valid options are: `csv`, `feather`.
+            :Default: `feather`.
 
         Returns
         -------
-        list[pl.DataFrame]
+        list[EventDataFrame]
             List of event dataframes.
 
         Raises
         ------
         AttributeError
             If `fileinfo` is None or the `fileinfo` dataframe is empty.
+        ValueError
+            If extension is not in list of valid extensions.
         """
         self._check_fileinfo()
 
-        event_dfs: list[pl.DataFrame] = []
+        event_dfs: list[EventDataFrame] = []
 
         # read and preprocess input files
         for fileinfo in tqdm(self.fileinfo.to_dicts()):
             filepath = Path(fileinfo['filepath'])
             filepath = self.raw_rootpath / filepath
 
-            filepath = self._raw_to_event_filepath(filepath, events_dirname=events_dirname)
-            event_df = pl.read_ipc(filepath)
+            filepath = self._raw_to_event_filepath(
+                filepath, events_dirname=events_dirname,
+                extension=extension,
+            )
+
+            if extension == 'feather':
+                event_df = pl.read_ipc(filepath)
+            elif extension == 'csv':
+                event_df = pl.read_csv(filepath)
+            else:
+                valid_extensions = ['csv', 'feather']
+                raise ValueError(
+                    f'unsupported file format "{extension}".'
+                    f'Supported formats are: {valid_extensions}',
+                )
 
             # Add fileinfo columns to dataframe.
             event_df = self._add_fileinfo(event_df, fileinfo)
 
-            event_dfs.append(event_df)
+            event_dfs.append(EventDataFrame(event_df))
 
         return event_dfs
 
@@ -373,26 +413,10 @@ class Dataset:
             if experiment is None.
         """
         self._check_gaze_dataframe()
-        self._check_experiment()
 
         disable_progressbar = not verbose
-
-        pix_position_columns = self.pix_position_columns
-        dva_position_columns = self._pixel_to_dva_position_columns(pix_position_columns)
-
-        for file_id, file_df in enumerate(tqdm(self.gaze, disable=disable_progressbar)):
-            pixel_positions = file_df.select(pix_position_columns)
-
-            dva_positions = self.experiment.screen.pix2deg(  # type: ignore[union-attr]
-                pixel_positions.to_numpy(),
-            )
-
-            self.gaze[file_id] = self.gaze[file_id].with_columns(
-                [
-                    pl.Series(name=dva_column_name, values=dva_positions[:, dva_column_id])
-                    for dva_column_id, dva_column_name in enumerate(dva_position_columns)
-                ],
-            )
+        for gaze_df in tqdm(self.gaze, disable=disable_progressbar):
+            gaze_df.pix2deg()
 
     def pos2vel(self, method: str = 'smooth', verbose: bool = True, **kwargs) -> None:
         """Compute gaze velocites in dva/s from dva coordinates.
@@ -417,31 +441,15 @@ class Dataset:
             if experiment is None.
         """
         self._check_gaze_dataframe()
-        self._check_experiment()
 
         disable_progressbar = not verbose
-
-        position_columns = self.dva_position_columns
-        velocity_columns = self._position_to_velocity_columns(position_columns)
-
-        for file_id, file_df in enumerate(tqdm(self.gaze, disable=disable_progressbar)):
-            positions = file_df.select(position_columns)
-
-            velocities = self.experiment.pos2vel(  # type: ignore[union-attr]
-                positions.to_numpy(), method=method, **kwargs,
-            )
-
-            self.gaze[file_id] = self.gaze[file_id].with_columns(
-                [
-                    pl.Series(name=velocity_column_name, values=velocities[:, column_id])
-                    for column_id, velocity_column_name in enumerate(velocity_columns)
-                ],
-            )
+        for gaze_df in tqdm(self.gaze, disable=disable_progressbar):
+            gaze_df.pos2vel(method=method, **kwargs)
 
     def detect_events(
             self,
             method: EventDetectionCallable,
-            eye: str = 'auto',
+            eye: str | None = 'auto',
             clear: bool = False,
             verbose: bool = True,
             **kwargs,
@@ -453,7 +461,7 @@ class Dataset:
         method : EventDetectionCallable
             The event detection method to be applied.
         eye : str
-            Select which eye to choose. Valid options are ``auto``, ``left``, ``right`` or ``eye``.
+            Select which eye to choose. Valid options are ``auto``, ``left``, ``right`` or ``None``.
             If ``auto`` is passed, eye is inferred in the order ``['right', 'left', 'eye']`` from
             the available :py:attr:`~.Dataset.gaze` dataframe columns.
         clear : bool
@@ -473,20 +481,24 @@ class Dataset:
 
         # Automatically infer eye to use for event detection.
         if eye == 'auto':
-            if 'x_right_dva' in self.gaze[0].columns:
+            if 'x_right_pos' in self.gaze[0].columns:
                 eye = 'right'
-            elif 'x_left_dva' in self.gaze[0].columns:
+            elif 'x_left_pos' in self.gaze[0].columns:
                 eye = 'left'
-            elif 'x_eye_dva' in self.gaze[0].columns:
-                eye = 'eye'
+            elif 'x_pos' in self.gaze[0].columns:
+                eye = None
             else:
                 raise AttributeError(
                     'Either right or left eye columns must be present in gaze data frame.'
                     f' Available columns are: {self.gaze[0].columns}',
                 )
 
-        position_columns = [f'x_{eye}_dva', f'y_{eye}_dva']
-        velocity_columns = [f'x_{eye}_vel', f'y_{eye}_vel']
+        if eye is None:
+            position_columns = ['x_pos', 'y_pos']
+            velocity_columns = ['x_vel', 'y_vel']
+        else:
+            position_columns = [f'x_{eye}_pos', f'y_{eye}_pos']
+            velocity_columns = [f'x_{eye}_vel', f'y_{eye}_vel']
 
         if not set(position_columns).issubset(set(self.gaze[0].columns)):
             raise AttributeError(
@@ -497,18 +509,21 @@ class Dataset:
 
         disable_progressbar = not verbose
 
-        event_dfs: list[pl.DataFrame] = []
+        event_dfs: list[EventDataFrame] = []
 
         for gaze_df, fileinfo in tqdm(
                 zip(self.gaze, self.fileinfo.to_dicts()), disable=disable_progressbar,
         ):
 
-            positions = gaze_df.select(position_columns).to_numpy()
-            velocities = gaze_df.select(velocity_columns).to_numpy()
+            positions = gaze_df.frame.select(position_columns).to_numpy()
+            velocities = gaze_df.frame.select(velocity_columns).to_numpy()
+            timesteps = gaze_df.frame.select('time').to_numpy()
 
-            event_df = method(positions=positions, velocities=velocities, **kwargs)
-            event_df = self._add_fileinfo(event_df, fileinfo)
+            event_df = method(
+                positions=positions, velocities=velocities, timesteps=timesteps, **kwargs,
+            )
 
+            event_df.frame = self._add_fileinfo(event_df.frame, fileinfo)
             event_dfs.append(event_df)
 
         if not self.events or clear:
@@ -516,28 +531,62 @@ class Dataset:
             return
 
         for file_id, event_df in enumerate(event_dfs):
-            self.events[file_id] = pl.concat(
-                [self.events[file_id], event_df],
+            self.events[file_id].frame = pl.concat(
+                [self.events[file_id].frame, event_df.frame],
                 how='diagonal',
             )
 
+    def compute_event_properties(
+            self,
+            event_properties: str | list[str],
+            verbose: bool = True,
+    ) -> None:
+        """Calculate an event property for and add it as a column to the event dataframe.
+
+        Parameters
+        ----------
+        event_properties:
+            The event properties to compute.
+        verbose : bool
+            If ``True``, show progress bar.
+
+        Raises
+        ------
+        InvalidProperty
+            If ``property_name`` is not a valid property. See
+            :py:mod:`pymovements.events.event_properties` for an overview of supported properties.
+        """
+        processor = EventGazeProcessor(event_properties)
+
+        identifier_columns = [column for column in self.fileinfo.columns if column != 'filepath']
+
+        disable_progressbar = not verbose
+        for events, gaze in tqdm(zip(self.events, self.gaze), disable=disable_progressbar):
+            new_properties = processor.process(events, gaze, identifiers=identifier_columns)
+
+            new_properties = new_properties.drop(identifier_columns)
+            new_properties = new_properties.drop(['name', 'onset', 'offset'])
+
+            events.add_event_properties(new_properties)
+
     def clear_events(self) -> None:
         """Clear event DataFrame."""
         if len(self.events) == 0:
             return
 
         for file_id, _ in enumerate(self.events):
-            self.events[file_id] = pl.DataFrame(schema=Event.schema)
+            self.events[file_id] = EventDataFrame()
 
     def save(
             self,
             events_dirname: str | None = None,
             preprocessed_dirname: str | None = None,
             verbose: int = 1,
+            extension: str = 'feather',
     ):
         """Save preprocessed gaze and event files.
 
-        Data will be saved as feather files to ``Dataset.preprocessed_roothpath`` or
+        Data will be saved as feather/csv files to ``Dataset.preprocessed_roothpath`` or
         ``Dataset.events_roothpath`` with the same directory structure as the raw data.
 
         Parameters
@@ -552,11 +601,17 @@ class Dataset:
             :py:meth:`pymovements.Dataset.preprocessed_rootpath`.
         verbose : int
             Verbosity level (0: no print output, 1: show progress bar, 2: print saved filepaths)
+        extension:
+            extension specifies the fileformat to store the data
         """
-        self.save_events(events_dirname, verbose=verbose)
-        self.save_preprocessed(preprocessed_dirname, verbose=verbose)
+        self.save_events(events_dirname, verbose=verbose, extension=extension)
+        self.save_preprocessed(preprocessed_dirname, verbose=verbose, extension=extension)
 
-    def save_events(self, events_dirname: str | None = None, verbose: int = 1):
+    def save_events(
+        self, events_dirname: str | None = None,
+        verbose: int = 1,
+        extension: str = 'feather',
+    ):
         """Save events to files.
 
         Data will be saved as feather files to ``Dataset.events_roothpath`` with the same directory
@@ -570,6 +625,14 @@ class Dataset:
             :py:meth:`pymovements.Dataset.events_rootpath`.
         verbose : int
             Verbosity level (0: no print output, 1: show progress bar, 2: print saved filepaths)
+        extension:
+            Specifies the file format for loading data. Valid options are: `csv`, `feather`.
+            :Default: `feather`.
+
+        Raises
+        ------
+        ValueError
+            If extension is not in list of valid extensions.
         """
         disable_progressbar = not verbose
 
@@ -577,19 +640,34 @@ class Dataset:
             raw_filepath = self.raw_rootpath / Path(self.fileinfo[file_id, 'filepath'])
             events_filepath = self._raw_to_event_filepath(
                 raw_filepath, events_dirname=events_dirname,
+                extension=extension,
             )
 
-            for column in event_df.columns:
+            event_df_out = event_df.frame.clone()
+            for column in event_df_out.columns:
                 if column in self.fileinfo.columns:
-                    event_df = event_df.drop(column)
+                    event_df_out = event_df_out.drop(column)
 
             if verbose >= 2:
                 print('Save file to', events_filepath)
 
             events_filepath.parent.mkdir(parents=True, exist_ok=True)
-            event_df.write_ipc(events_filepath)
+            if extension == 'feather':
+                event_df_out.write_ipc(events_filepath)
+            elif extension == 'csv':
+                event_df_out.write_csv(events_filepath)
+            else:
+                valid_extensions = ['csv', 'feather']
+                raise ValueError(
+                    f'unsupported file format "{extension}".'
+                    f'Supported formats are: {valid_extensions}',
+                )
 
-    def save_preprocessed(self, preprocessed_dirname: str | None = None, verbose: int = 1):
+    def save_preprocessed(
+        self, preprocessed_dirname: str | None = None,
+        verbose: int = 1,
+        extension: str = 'feather',
+    ):
         """Save preprocessed gaze files.
 
         Data will be saved as feather files to ``Dataset.preprocessed_roothpath`` with the same
@@ -603,6 +681,14 @@ class Dataset:
             :py:meth:`pymovements.Dataset.preprocessed_rootpath`.
         verbose : int
             Verbosity level (0: no print output, 1: show progress bar, 2: print saved filepaths)
+        extension:
+            Specifies the file format for loading data. Valid options are: `csv`, `feather`.
+            :Default: `feather`.
+
+        Raises
+        ------
+        ValueError
+            If extension is not in list of valid extensions.
         """
         disable_progressbar = not verbose
 
@@ -610,17 +696,28 @@ class Dataset:
             raw_filepath = self.raw_rootpath / Path(self.fileinfo[file_id, 'filepath'])
             preprocessed_filepath = self._raw_to_preprocessed_filepath(
                 raw_filepath, preprocessed_dirname=preprocessed_dirname,
+                extension=extension,
             )
 
+            gaze_df_out = gaze_df.frame.clone()
             for column in gaze_df.columns:
                 if column in self.fileinfo.columns:
-                    gaze_df = gaze_df.drop(column)
+                    gaze_df_out = gaze_df_out.drop(column)
 
             if verbose >= 2:
                 print('Save file to', preprocessed_filepath)
 
             preprocessed_filepath.parent.mkdir(parents=True, exist_ok=True)
-            gaze_df.write_ipc(preprocessed_filepath)
+            if extension == 'feather':
+                gaze_df_out.write_ipc(preprocessed_filepath)
+            elif extension == 'csv':
+                gaze_df_out.write_csv(preprocessed_filepath)
+            else:
+                valid_extensions = ['csv', 'feather']
+                raise ValueError(
+                    f'unsupported file format "{extension}".'
+                    f'Supported formats are: {valid_extensions}',
+                )
 
     @property
     def path(self) -> Path:
@@ -749,33 +846,6 @@ class Dataset:
         """
         return self.path / self.raw_dirname
 
-    @property
-    def pix_position_columns(self) -> list[str]:
-        """Get pixel position columns for this dataset."""
-        pix_position_columns = [
-            'x_left_pix', 'y_left_pix', 'x_right_pix', 'y_right_pix', 'x_eye_pix', 'y_eye_pix',
-        ]
-        pix_position_columns = list(set(pix_position_columns) & set(self.gaze[0].columns))
-        return pix_position_columns
-
-    @property
-    def dva_position_columns(self) -> list[str]:
-        """Get dva position columns for this dataset."""
-        dva_position_columns = [
-            'x_left_dva', 'y_left_dva', 'x_right_dva', 'y_right_dva', 'x_eye_dva', 'y_eye_dva',
-        ]
-        dva_position_columns = list(set(dva_position_columns) & set(self.gaze[0].columns))
-        return dva_position_columns
-
-    @property
-    def velocity_columns(self) -> list[str]:
-        """Get velocity columns for this dataset."""
-        velocity_columns = [
-            'x_left_vel', 'y_left_vel', 'x_right_vel', 'y_right_vel', 'x_eye_vel', 'y_eye_vel',
-        ]
-        velocity_columns = list(set(velocity_columns) & set(self.gaze[0].columns))
-        return velocity_columns
-
     def _check_fileinfo(self) -> None:
         """Check if fileinfo attribute is set and there is at least one row present."""
         if self.fileinfo is None:
@@ -792,15 +862,11 @@ class Dataset:
         if len(self.gaze) == 0:
             raise AttributeError('no files present in gaze attribute')
 
-    def _check_experiment(self) -> None:
-        """Check if experiment attribute has been set."""
-        if self.experiment is None:
-            raise AttributeError('experiment must be specified for this method to work.')
-
     def _raw_to_preprocessed_filepath(
             self,
             raw_filepath: Path,
             preprocessed_dirname: str | None = None,
+            extension: str = 'feather',
     ) -> Path:
         """Get preprocessed filepath in accordance to filepath of the raw file.
 
@@ -815,6 +881,8 @@ class Dataset:
             :py:meth:`pymovements.Dataset.path`.
             This argument is used only for this single call and does not alter
             :py:meth:`pymovements.Dataset.preprocessed_rootpath`.
+        extension:
+            extension specifies the fileformat to store the data
 
         Returns
         -------
@@ -832,14 +900,19 @@ class Dataset:
         preprocessed_file_dirpath = preprocessed_rootpath / relative_raw_dirpath
 
         # Get new filename for saved feather file.
-        preprocessed_filename = raw_filepath.stem + '.feather'
+        preprocessed_filename = raw_filepath.stem + '.' + extension
 
         return preprocessed_file_dirpath / preprocessed_filename
 
-    def _raw_to_event_filepath(self, raw_filepath: Path, events_dirname: str | None = None) -> Path:
+    def _raw_to_event_filepath(
+        self,
+        raw_filepath: Path,
+        events_dirname: str | None = None,
+        extension: str = 'feather',
+    ) -> Path:
         """Get event filepath in accordance to filepath of the raw file.
 
-        The event filepath will point to a feather file.
+        The event filepath will point to file with the specified extension.
 
         Parameters
         ----------
@@ -850,6 +923,8 @@ class Dataset:
             :py:meth:`pymovements.Dataset.path`.
             This argument is used only for this single call and does not alter
             :py:meth:`pymovements.Dataset.events_rootpath`.
+        extension:
+            extension specifies the fileformat to store the data
 
         Returns
         -------
@@ -867,26 +942,10 @@ class Dataset:
         events_file_dirpath = events_rootpath / relative_raw_dirpath
 
         # Get new filename for saved feather file.
-        events_filename = raw_filepath.stem + '.feather'
+        events_filename = raw_filepath.stem + '.' + extension
 
         return events_file_dirpath / events_filename
 
-    def _pixel_to_dva_position_columns(self, columns: list[str]) -> list[str]:
-        """Get corresponding dva position columns from pixel position columns."""
-        return [
-            column.replace('_pix', '_dva')
-            for column in columns
-            if column.endswith('_pix')
-        ]
-
-    def _position_to_velocity_columns(self, columns: list[str]) -> list[str]:
-        """Get corresponding velocity columns from dva position columns."""
-        return [
-            column.replace('_dva', '_vel')
-            for column in columns
-            if column.endswith('_dva')
-        ]
-
     def _add_fileinfo(self, df: pl.DataFrame, fileinfo: dict[str, Any]) -> pl.DataFrame:
         """Add columns from fileinfo to dataframe.
 
