@@ -26,8 +26,8 @@ from pathlib import Path
 from typing import Any
 from urllib.error import URLError
 
-from pymovements.base import Experiment
 from pymovements.datasets.dataset import Dataset
+from pymovements.gaze.experiment import Experiment
 from pymovements.utils.archives import extract_archive
 from pymovements.utils.downloads import download_file
 
@@ -157,7 +157,8 @@ class PublicDataset(Dataset, metaclass=ABCMeta):
                     )
                     success = True
 
-                except URLError as error:
+                # pylint: disable=overlapping-except
+                except (URLError, OSError, RuntimeError) as error:
                     print(f'Failed to download (trying next):\n{error}')
                     # downloading the resource, try next mirror
                     continue
